#pragma once

#include "state.h"
#include "tools/music.h"
#include "tools/resourceholder.h"
#include <SFML/Graphics/RenderWindow.hpp>

class BPMCalculator;
class Group;
class Editor;

class EditorState : public GUIState
{
public:

    struct Callbacks
    {
        std::function<void(void)> onLeaveEditorState;
    };

    explicit EditorState(sf::RenderWindow& game_window, std::unique_ptr<Editor>&& editor, Callbacks&& callbacks, const FontHolder& font_holder);
    virtual ~EditorState() override;
    virtual void input(const sf::Event& event) override;
    virtual void update(const sf::Time& dt) override;
    virtual void draw() const override;

    virtual void enter() override;
    virtual void leave() override;

private:
    const std::shared_ptr<sf::Font> _font;
    sf::RenderWindow& _game_window;

    Callbacks _callbacks;

    Music _music;
    std::shared_ptr<BPMCalculator> _bpm_calculator;
    std::shared_ptr<Group> _group;

    std::unique_ptr<Editor> _editor;
};

